#!/usr/bin/env python3
# fileencoding=utf-8

'''对 AF_UNIX socket 的操作'''

import os
import socket
import struct

class socket_c(socket.socket):
  '''client 端 socket'''
  def __init__(self, socketFile):
    socket.socket.__init__(self, socket.AF_UNIX)
    self.connect(socketFile)

  def __del__(self):
    self.close()

  def send(self, msg):
    '''发送消息'''
    if isinstance(msg, int):
      msg = struct.pack('<I', msg)
      self._send(msg)
    elif isinstance(msg, str):
      msg = msg.encode()
      self.send(len(msg))
      self._send(msg)
    elif isinstance(msg, bytes):
      self._send(msg)
    else:
      raise TypeError('不支持发送的类型: '+type(msg).__name__)

  def recv(self, type, encoding=None):
    '''接收数据，对于字符串和 bytes，先接收大小'''
    if type == int:
      size = 4
    else:
      size = self.recv(int)
    msg = socket.socket.recv(self, size)
    if type == int:
      msg = struct.unpack('<I', msg)[0]
    elif type == str:
      if encoding:
        msg = msg.decode(encoding)
      else:
        msg = msg.decode()

    return msg

  def _send(self, Bytes):
    '''发送 bytes'''
    if not isinstance(Bytes, bytes):
      raise TypeError('只能发送 bytes 类型数据')
    # print('sent:', Bytes)
    socket.socket.send(self, Bytes)

class socket_s(socket.socket):
  '''server 端 socket'''
  def __init__(self, socketFile, force=False, backlog=5):
    socket.socket.__init__(self, socket.AF_UNIX)
    if force and os.path.exists(socketFile):
        os.unlink(socketFile)
    self.bind(socketFile)
    self.listen(backlog)

  def accept(self):
    self.connection, self.address = socket.socket.accept(self)

  def send(self, msg):
    '''发送消息'''
    if isinstance(msg, int):
      msg = struct.pack('<I', msg)
      self._send(msg)
    elif isinstance(msg, str):
      msg = msg.encode()
      self.send(len(msg))
      self._send(msg)
    elif isinstance(msg, bytes):
      self._send(msg)
    else:
      raise TypeError('不支持发送的类型: '+type(msg).__name__)

  def recv(self, type, encoding=None):
    '''接收数据，对于字符串和 bytes，先接收大小'''
    if type == int:
      size = 4
    else:
      size = self.recv(int)
    msg = socket.socket.recv(self.connection, size)
    if type == int:
      msg = struct.unpack('<I', msg)[0]
    elif type == str:
      if encoding:
        msg = msg.decode(encoding)
      else:
        msg = msg.decode()

    return msg

  def _send(self, Bytes):
    '''发送 bytes'''
    if not isinstance(Bytes, bytes):
      raise TypeError('只能发送 bytes 类型数据')
    # print('sent:', Bytes)
    socket.socket.send(self.connection, Bytes)

  def __del__(self):
    try:
      self.connection.close()
    except AttributeError:
      pass

